/*
 *  Project   : tin - a threaded Netnews reader
 *  Module    : active.c
 *  Author    : I.Lea
 *  Created   : 16-02-92
 *  Updated   : 07-08-92
 *  Notes     :
 *  Copyright : (c) Copyright 1991-92 by Iain Lea
 *              You may  freely  copy or  redistribute  this software,
 *              so  long as there is no profit made from its use, sale
 *              trade or  reproduction.  You may not change this copy-
 *              right notice, and it must be included in any copy made
 */

#include	"tin.h"

char new_active_file_server[PATH_LEN];
char new_active_file_attribute[32];
int group_hash[TABLE_SIZE];			/* group name --> active[] */
int reread_active_file = FALSE;
int active_index = -1;

/*
 *  Resync active file when SIGALRM signal received that
 *  is triggered by alarm (reread_active_file_secs) call.
 */

void resync_active_file ()
{
	if (reread_active_file) {
		free_active_arrays ();
		max_active = DEFAULT_ACTIVE_NUM;
		expand_active ();
		read_active_file ();
		read_attributes_file ();
		read_newsgroups_file ();
		read_newsrc (TRUE);
		toggle_my_groups (show_only_unread_groups);
		set_groupname_len (FALSE);
		set_alarm_signal ();
		mail_setup ();
		group_selection_page ();
	}
}

/*
 *  Find group name in active[] array and return index otherwise -1
 */
 
int find_group_index (group)
	char *group;
{
	int i = -1;
	long h;

	h = hash_groupname (group);

	i = group_hash[h];
	
	/* 
	 * hash linked list chaining 
	 */
	while (i >= 0) {
		if (strcmp (active[i].name, group) == 0) {
			return (i);
		}
		i = active[i].next;
	}

	return (-1);
}

/*
 *  Load the active file into active[] and create copy of active ~/.tin/active
 */

void read_active_file ()
{
	FILE *fp;
	char *p, *q, *r;
	char buf[LEN];
	char moderated = 'y';
	int i;
	long h;
		
	num_active = 0;

	if ((update && update_fork) || ! update) {
		wait_message (txt_reading_active_file);
	}

	if ((fp = open_active_fp ()) == NULL) {
		if (compiled_with_nntp) {
			sprintf (msg, txt_cannot_open_active_file, active_file, progname);
			wait_message (msg);
		} else {
			fputc ('\n', stderr);
			fprintf (stderr, txt_cannot_open, active_file);
			fputc ('\n', stderr);
			fflush (stderr);
		}
		exit (1);
	}

	for (i = 0; i < TABLE_SIZE; i++) {
		group_hash[i] = -1;
	}

	while (fgets (buf, sizeof (buf), fp) != NULL) {
		for (p = buf; *p && *p != ' '; p++)
			continue;
		if (*p != ' ') {
			error_message (txt_bad_active_file, buf);
			continue;
		}
		*p++ = '\0';

		if (num_active >= max_active) {
			debug_nntp ("read_active_file", "EXPANDING active file");
			expand_active ();
		}

		h = hash_groupname (buf);

		if (group_hash[h] == -1) {
			group_hash[h] = num_active;
		} else {				/* hash linked list chaining */
			for (i=group_hash[h]; active[i].next >= 0; i=active[i].next) {
				if (strcmp(active[i].name, buf) == 0) {
					goto read_active_continue;		/* kill dups */
				}
			}
			if (strcmp(active[i].name, buf) == 0)
				goto read_active_continue;
			active[i].next = num_active;
		}

		for (q = p; *q && *q != ' '; q++)
			continue;
		if (*q != ' ') {
			error_message (txt_bad_active_file, buf);
			continue;
		}
		*q++ = '\0';

		for (r = q; *r && *r != '\n'; r++) {
			if (*r == 'y' || *r == 'm') {
				moderated = *r;
				break;
			}
		}

		/*
		 * Group info.
		 */
		active[num_active].name = str_dup (buf);
		active[num_active].description = (char *) 0;
		active[num_active].max = (long) atol (p);
		active[num_active].min = (long) atol (q);
		active[num_active].moderated = moderated;
		active[num_active].next = -1;			/* hash chaining */
		active[num_active].my_group = UNSUBSCRIBED;	/* not in my_group[] yet */
		active[num_active].unread = 0;

		num_active++;

read_active_continue:;

	}
	fclose (fp);

	/*
	 *  exit if active file is empty
	 */
	if (! num_active) {
		error_message (txt_active_file_is_empty, active_file);
		exit (1);
	}

	if ((cmd_line && ! update && ! verbose) || (update && update_fork)) {
/* || ! update && ! verbose) { */
		wait_message ("\n");
	}

	check_for_any_new_groups ();
}

/*
 *  create ~/.tin/active if it does not exist (local news only) 
 */

void backup_active (create)
	int create;
{
	char buf[PATH_LEN];
	FILE *fp;
	register int i;
	struct stat sb;
	
	sprintf (buf, "%s/active", rcdir);
	
	if (create) {
		if (stat (buf, &sb) != -1) {
			return;
		}
	}
	
	if ((fp = fopen (buf, "w")) != (FILE *) 0) {
		for (i = 0; i < num_active ; i++) {	/* for each group */
			fprintf (fp, "%s\n", active[i].name);
		}
		fclose (fp);
		chmod (buf, 0644);
	}
}

/*
 *  check for any newly created newsgroups.
 *
 * If reading news locally stat() the active file to get its
 * size otherwise do a LIST NEWGROUPS to the NNTP server
 */

void check_for_any_new_groups ()
{
	char old_active_file_server[LEN];
	char old_active_file_attribute[LEN];
	char buf[LEN];
	FILE *fp;
	int group_not_found;
	int ch, num = 0;
	int ch_default = 'n';
	int update_old_active = FALSE;
	int max_old_active;
	int new_active_size;
	int old_active_size;
	long epoch;
	register int i, j;
	struct stat sb;
	struct tm *tm;
	struct notify_t {
		char name[PATH_LEN];
		int len;
		int visited;
	} *old_active = (struct notify_t *) 0;

	if (! check_for_new_newsgroups) {
		return;
	}
	
	/*
	 * reading news locally (local) or via NNTP (server name)
	 */	
	if (read_news_via_nntp) {
		time (&epoch);
		tm = localtime (&epoch);
		sprintf (new_active_file_attribute, "%02d%02d%02d %02d%02d%02d",
			tm->tm_year, tm->tm_mon+1, tm->tm_mday, 
			tm->tm_hour, tm->tm_min, tm->tm_sec);
		strcpy (new_active_file_server, nntp_server);
		max_old_active = 16;
	} else {
		backup_active (TRUE);
		strcpy (new_active_file_server, "local");
		max_old_active = num_active;
		if (stat (active_file, &sb) >= 0) {
			sprintf (new_active_file_attribute, "%d", sb.st_size);
		}
	}
	
	/*
	 * find out if we have read news from here before otherwise -1
	 */
	active_index = find_active_size_index (new_active_file_server);
		
	if (debug == 2) {
		if (active_index >= 0) {	
			strcpy (old_active_file_server, active_size[active_index].server); 
			strcpy (old_active_file_attribute, active_size[active_index].attribute);
		} else {
			strcpy (old_active_file_server, "UNKNOWN"); 
			strcpy (old_active_file_attribute, "UNKNOWN");
		}	
		sprintf (msg, "Active size index=[%d]  old=[%s %s]  new=[%s %s]", 
			active_index,
			old_active_file_server, old_active_file_attribute,
			new_active_file_server, new_active_file_attribute);
		error_message (msg, "");						
		sleep (2);
	}
			
	if (! read_news_via_nntp && active_index >= 0) {
		new_active_size = atoi (new_active_file_attribute);
		old_active_size = atoi (active_size[active_index].attribute);
		if (new_active_size <= old_active_size) {
			goto notify_groups_done;
		}
	}

	if ((fp = open_newgroups_fp (active_index)) == (FILE *) 0) {
		goto notify_groups_done;
	}

	Raw (TRUE);

	old_active = (struct notify_t *) my_malloc ((unsigned) sizeof (struct notify_t) * max_old_active);
	if (old_active == (struct notify_t *) 0) {
		error_message (txt_out_of_memory, progname);
		goto notify_groups_done;
	}
	
	while (fgets (buf, sizeof (buf), fp) != NULL) {
		if (buf[0] != '.') {
			strncpy (old_active[num].name, buf, sizeof (old_active[num].name));		
			old_active[num].len = strlen (old_active[num].name)-1;
			old_active[num].name[old_active[num].len] = '\0';
			old_active[num].visited = FALSE;
			num++;
			if (num >= max_old_active) {
				max_old_active= max_old_active + (max_old_active / 2);
				old_active= (struct notify_t*) my_realloc(
					(char *) old_active, 
					(unsigned) sizeof(struct notify_t) * max_old_active);
				if (old_active == (struct notify_t *) 0) {
					error_message (txt_out_of_memory, progname);
					goto notify_groups_done;
				}
			}
		}	
	}

	fclose (fp);

	if (read_news_via_nntp) {
		for (i = 0 ; i < num ; i++) {	
			if (! prompt_subscribe_group (old_active[i].name)) {
				goto notify_groups_done;
			}
		}
	} else {	
		wait_message (txt_checking_active_file);
		for (i = 0 ; i < num_active ; i++) {	
			group_not_found = TRUE;
			for (j=0; j < num ; j++) {
				if (strcmp (old_active[j].name, active[i].name) == 0) {
					group_not_found = FALSE;	/* found it so read in next group */
					old_active[j].visited = TRUE;
					break;
				}
			}

			if (group_not_found == FALSE) {
				continue;
			}	
	
			update_old_active = TRUE;
	
			if (! prompt_subscribe_group (active[i].name)) {
				goto notify_groups_done;
			}
		}
	}
	if (cmd_line) {
		fputc ('\r', stdout);
		fflush (stdout);
	}
	CleartoEOLN();

	if (! read_news_via_nntp) {
		/*
		 * Look for bogus groups 
		 */
		ch_default = 'y';
		for (j = 0 ; j < num ; j++)  {
			if (old_active[j].visited) {
				continue;
			}
			do {	
				update_old_active= 1;
				fputc ('\r', stdout);
				CleartoEOLN ();
				printf (txt_delete_bogus_group, old_active[j].name, ch_default);
				fflush (stdout);
				ch = ReadCh ();
				if (ch == CR) {
					ch = ch_default;
				}
			} while (ch != 'y' && ch != 'n' && ch != 'q');
	
			switch (ch) {
				case 'y':
					delete_group (old_active[j].name);
					break;	   
				case 'q':
					goto notify_groups_done;	   
				case 'n':
				default:
					break;	   
			}		
			printf ("\r\n");
		}
	}
	
	/*
	 *  write active[] to ~/.tin/active (local spooldir)
	 */
	if (! read_news_via_nntp && update_old_active) {
		backup_active (FALSE);
	}

notify_groups_done:
	
	/*
	 * update attribute field/create new entry with new size/date
	 */
	if (active_index >= 0) {
		if (active_size[active_index].attribute != (char *) 0) {
			free (active_size[active_index].attribute);
			active_size[active_index].attribute = (char *) 0;
		}
		active_size[active_index].attribute = 
			str_dup (new_active_file_attribute);
	} else {
		sprintf (buf, "%s[%s]", new_active_file_server, new_active_file_attribute);
		load_active_size_info (buf);
	}

	if (old_active != (struct notify_t *) 0) {
		free ((char *) old_active);
		old_active = (struct notify_t *) 0;
	}

	if (cmd_line) {
		Raw (FALSE);
	}
}

/*
 * prompt user if new group should be subscribed to
 */
 
int prompt_subscribe_group (group)
	char *group;
{
	int ch, ch_default = 'n';
	int idx;
	
	do {
		if (cmd_line) {
			fputc ('\r', stdout);
		}
		CleartoEOLN ();
		printf (txt_subscribe_to_new_group, group, ch_default);
		fflush (stdout);
		ch = ReadCh ();
		if (ch == CR) {
			ch = ch_default;
		}
	} while (! strchr ("nqy\033", ch));
		
	fputc (ch, stdout);
	
	switch (ch) {
		case 'y':
			idx = add_group (group, TRUE);
			subscribe (active[my_group[idx]].name, ':',
				   my_group[idx], FALSE);
			break;	   
		case 'q':
		case ESC:
			return FALSE;			   
		case 'n':
		default:
			break;	   
	}	
		
	if (cmd_line) {
		printf ("\r\n%s", txt_checking);
		fflush (stdout);
	} else {	
		wait_message (txt_checking);
	}

	return TRUE;			   
}

/*
 * Per group attributes
 */
 
void set_default_attributes ()
{
#ifndef INDEX_DAEMON
	register int i;
	
	for (i = 0; i < num_active ; i++) {
		active[i].attribute.maildir = default_maildir;
		active[i].attribute.savedir = default_savedir;
		active[i].attribute.sigfile = default_sigfile;
		active[i].attribute.followup_to = (char *) 0;
		active[i].attribute.read_during_session = FALSE;
		active[i].attribute.show_only_unread = default_show_only_unread;
		active[i].attribute.thread_arts = default_thread_arts;
		active[i].attribute.sort_art_type = default_sort_art_type;
		active[i].attribute.show_author  = default_show_author;
		active[i].attribute.auto_save= default_auto_save;
		active[i].attribute.batch_save = default_batch_save;
		active[i].attribute.post_proc_type = default_post_proc_type; 
	}
#endif	/* INDEX_DAEMON */
}

/*
 *  Load the group attributes into active[].attribute from ~/.tin/attributes 
 *
 *  attribute.maildir          = STRING
 *  attribute.savedir          = STRING
 *  attribute.sigfile          = STRING
 *  attribute.followup_to      = STRING
 *  attribute.auto_save        = ON/OFF
 *  attribute.batch_save       = ON/OFF
 *  attribute.show_only_unread = ON/OFF
 *  attribute.thread_arts      = ON/OFF
 *  attribute.sort_art_type    = NUM
 *    0=??, 1=??, 2=??, 3=??, 4=??
 *  attribute.show_author      = NUM
 *    
 *  attribute.post_proc_type   = NUM
 *  
 */

void read_attributes_file ()
{
#ifndef INDEX_DAEMON

	char buf[PATH_LEN];
	char line[PATH_LEN];
	FILE *fp;
	int num;
	int index = -1;
		
	set_default_attributes ();

	if ((fp = fopen (attributes_file, "r")) == (FILE *) 0) {
		return;
	}

	if ((update && update_fork) || ! update) {
		wait_message (txt_reading_attributes_file);
	}

	while (fgets (line, sizeof (line), fp) != NULL) {
		if (line[0] == '#' || line[0] == '\n') {
			continue;
		}
		if (match_string (line, "newsgroup=", buf, sizeof (buf))) {
if (debug == 2) {
	error_message("group=[%s]",buf);
}
			index = find_group_index (buf);
			continue;
		}
		if (match_string (line, "maildir=", buf, sizeof (buf))) {
			if (index >= 0) {
				active[index].attribute.maildir = str_dup (buf);
if (debug == 2) {
	error_message("maildir=[%s]",active[index].attribute.maildir);
}
			}
			continue;
		}
		if (match_string (line, "savedir=", buf, sizeof (buf))) {
			if (index >= 0) {
				active[index].attribute.savedir = str_dup (buf);
if (debug == 2) {
	error_message("savedir=[%s]",active[index].attribute.savedir);
}
			}
			continue;
		}
		if (match_string (line, "sigfile=", buf, sizeof (buf))) {
			if (index >= 0) {
				active[index].attribute.sigfile = str_dup (buf);
if (debug == 2) {
	error_message("sigfile=[%s]",active[index].attribute.sigfile);
}
			}
			continue;
		}
		if (match_string (line, "followup_to=", buf, sizeof (buf))) {
			if (index >= 0) {
				active[index].attribute.followup_to = str_dup (buf);
if (debug == 2) {
	error_message("followup_to=[%s]",active[index].attribute.followup_to);
}
			}
			continue;
		}
		if (match_boolean (line, "show_only_unread=", &num)) {
			if (index >= 0) {
				active[index].attribute.show_only_unread = num;
			}
			continue;
		}
		if (match_boolean (line, "thread_arts=", &num)) {
			if (index >= 0) {
				active[index].attribute.thread_arts = num;
			}
			continue;
		}
		if (match_boolean (line, "auto_save=", &num)) {
			if (index >= 0) {
				active[index].attribute.auto_save = num;
			}
			continue;
		}
		if (match_boolean (line, "batch_save=", &num)) {
			if (index >= 0) {
				active[index].attribute.batch_save = num;
			}
			continue;
		}
		if (match_number (line, "sort_art_type=", &num)) {
			if (index >= 0) {
				active[index].attribute.sort_art_type = num;
			}
			continue;
		}
		if (match_number (line, "show_author=", &num)) {
			if (index >= 0) {
				active[index].attribute.show_author = num;
			}
			continue;
		}
		if (match_number (line, "post_proc_type=", &num)) {
			if (index >= 0) {
				active[index].attribute.post_proc_type = num;
			}
			continue;
		}
	}

	fclose (fp);

	if ((cmd_line && ! update && ! verbose) || (update && update_fork)) {
		wait_message ("\n");
	}

#endif	/* INDEX_DAEMON */
}

/*
 *  Save the group attributes from active[].attribute to ~/.tin/attributes 
 */

void write_attributes_file ()
{
#ifndef INDEX_DAEMON

#endif	/* INDEX_DAEMON */
}

/*
 *  Load the text description from LIBDIR/newsgroups for each group
 *  into the active[] array.
 */

void read_newsgroups_file ()
{
#ifndef INDEX_DAEMON

	FILE *fp;
	char *p, *q;
	char buf[LEN];
	char group[PATH_LEN];
	int i;
	
	if (! show_description || save_news) {
		if (update && ! update_fork) {
			return;
		}	
	}

	wait_message (txt_reading_newsgroups_file);

	if ((fp = open_newsgroups_fp ()) == (FILE *) 0) {
		return;
	}

	while (fgets (buf, sizeof (buf), fp) != NULL) {
		buf[strlen (buf)-1] = '\0';

		for (p = buf, q = group ; *p && *p != ' ' && *p != '\t' ; p++, q++) {
			*q = *p;
		}
		*q = '\0';

		while (*p == '\t' || *p == ' ') {
			p++;
		}	

		i = find_group_index (group);
		 
		if (i >= 0 && active[i].description == (char *) 0) {
			active[i].description = str_dup (p);
		}
	}
	fclose (fp);

	if (cmd_line && ! update && ! verbose) {
		wait_message ("\n");
	}
#endif	/* INDEX_DAEMON */
}


void load_active_size_info (info)
	char *info;
{
	char *ptr_name;
	char *ptr_size;
	char buf[PATH_LEN];
	int i;
		
	/*
	 * initialize active_size[] if no entries
	 */
	if (! num_active_size) {
		for (i = 0 ; i < max_active_size ; i++) {
			active_size[i].server = (char *) 0;
			active_size[i].attribute = (char *) 0;
		}
	}

	my_strncpy (buf, info, sizeof (buf));
	
	ptr_name = buf;

	ptr_name = strchr (buf, ']'); 
	if (*ptr_name) {
		*ptr_name = '\0';
	}
	ptr_name = buf;

	ptr_name = strchr (buf, '['); 
	if (*ptr_name) {
		ptr_size = ptr_name;
		*ptr_name = '\0';
		if (num_active_size > max_active_size) {
			expand_active_size ();
		}
		active_size[num_active_size].server = str_dup (buf);
		active_size[num_active_size].attribute = str_dup (++ptr_size);
		if (debug == 2) {
			sprintf (buf, "ACTIVE server=[%s] attribute=[%s]", 
				active_size[num_active_size].server,
				active_size[num_active_size].attribute);
			error_message ("%s", buf);
		}	
		num_active_size++;
	}
}


int find_active_size_index (cur_active_server)
	char *cur_active_server;
{
	int i, found = FALSE;
		
	for (i = 0 ; i < num_active_size ; i++) {
		if (strcmp (cur_active_server, active_size[i].server) == 0) {
			found = TRUE;
			break;
		}
  	}

	return (found ? i : -1);
}
